<?php

return [

    /*
    |--------------------------------------------------------------------------
    | PDO Fetch Style
    |--------------------------------------------------------------------------
    |
    | By default, database results will be returned as instances of the PHP
    | stdClass object; however, you may desire to retrieve records in an
    | array format for simplicity. Here you can tweak the fetch style.
    |
    */

    'fetch' => PDO::FETCH_CLASS,

    /*
    |--------------------------------------------------------------------------
    | Default Database Connection Name
    |--------------------------------------------------------------------------
    |
    | Here you may specify which of the database connections below you wish
    | to use as your default connection for all database work. Of course
    | you may use many connections at once using the Database library.
    |
    */

    'default' => env('DB_CONNECTION', 'mysql'),

    /*
    |--------------------------------------------------------------------------
    | Database Connections
    |--------------------------------------------------------------------------
    |
    | Here are each of the database connections setup for your application.
    | Of course, examples of configuring each database platform that is
    | supported by Laravel is shown below to make development simple.
    |
    |
    | All database work in Laravel is done through the PHP PDO facilities
    | so make sure you have the driver for your particular database of
    | choice installed on your machine before you begin development.
    |
    */

    'connections' => [

        'sqlite' => [
            'driver' => 'sqlite',
            'database' => env('DB_DATABASE', database_path('database.sqlite')),
            'prefix' => '',
        ],

        'mysql' => [
            'driver' => 'mysql',
            //'host' => env('DB_HOST', 'localhost'),
            'read' => [
                'host' => env('DB_READ_HOST', 'localhost'),
            ],
            'write' => [
                'host' => env('DB_WRITE_HOST', 'localhost'),
            ],
            'port' => env('DB_PORT', '3306'),
            'database' => env('DB_DATABASE', 'forge'),
            'username' => env('DB_USERNAME', 'forge'),
            'password' => env('DB_PASSWORD', ''),
            'charset' => 'utf8',
            'collation' => 'utf8_unicode_ci',
            'prefix' => '',
            'strict' => false,
            'engine' => null,
        ],
        'chapter_order_mysql' => [
            'driver' => 'mysql',
            'host' => env('CHAPTER_DB_HOST', 'localhost'),
            'port' => env('CHAPTER_DB_PORT', '3306'),
            'database' => env('CHAPTER_DB_DATABASE', 'forge'),
            'username' => env('CHAPTER_DB_USERNAME', 'forge'),
            'password' => env('CHAPTER_DB_PASSWORD', ''),
            'charset' => 'utf8',
            'collation' => 'utf8_unicode_ci',
            'prefix' => '',
            'strict' => false,
            'engine' => null,
        ],
        'qapp_order_mysql' => [
            'driver' => 'mysql',
            'host' => env('QAPP_DB_HOST', 'localhost'),
            'port' => env('QAPP_DB_PORT', '3306'),
            'database' => env('QAPP_DB_DATABASE', 'forge'),
            'username' => env('QAPP_DB_USERNAME', 'forge'),
            'password' => env('QAPP_DB_PASSWORD', ''),
            'charset' => 'utf8',
            'collation' => 'utf8_unicode_ci',
            'prefix' => '',
            'strict' => false,
            'engine' => null,
        ],
        'qapp_log_mysql' => [
            'driver' => 'mysql',
            'host' => env('QAPP_LOG_DB_HOST', 'localhost'),
            'port' => env('QAPP_LOG_DB_PORT', '3306'),
            'database' => env('QAPP_LOG_DB_DATABASE', 'forge'),
            'username' => env('QAPP_LOG_DB_USERNAME', 'forge'),
            'password' => env('QAPP_LOG_DB_PASSWORD', ''),
            'charset' => 'utf8',
            'collation' => 'utf8_unicode_ci',
            'prefix' => '',
            'strict' => false,
            'engine' => null,
        ],
        'new_yunqi' => [
            'driver' => 'mysql',
            'host' => env('NEW_YUNQI_DB_HOST', 'localhost'),
            'port' => env('NEW_YUNQI_DB_PORT', '3306'),
            'database' => env('NEW_YUNQI_DB_DATABASE', 'forget'),
            'username' => env('NEW_YUNQI_DB_USERNAME', 'forget'),
            'password' => env('NEW_YUNQI_DB_PASSWORD', ''),
            'charset' => 'utf8',
            'collation' => 'utf8_unicode_ci',
            'prefix' => '',
            'strict' => false,
            'engine' => null,
        ],
        'pgsql' => [
            'driver' => 'pgsql',
            'host' => env('DB_HOST', 'localhost'),
            'port' => env('DB_PORT', '5432'),
            'database' => env('DB_DATABASE', 'forge'),
            'username' => env('DB_USERNAME', 'forge'),
            'password' => env('DB_PASSWORD', ''),
            'charset' => 'utf8',
            'prefix' => '',
            'schema' => 'public',
        ],

    ],

    /*
    |--------------------------------------------------------------------------
    | Migration Repository Table
    |--------------------------------------------------------------------------
    |
    | This table keeps track of all the migrations that have already run for
    | your application. Using this information, we can determine which of
    | the migrations on disk haven't actually been run in the database.
    |
    */

    'migrations' => 'migrations',

    /*
    |--------------------------------------------------------------------------
    | Redis Databases
    |--------------------------------------------------------------------------
    |
    | Redis is an open source, fast, and advanced key-value store that also
    | provides a richer set of commands than a typical key-value systems
    | such as APC or Memcached. Laravel makes it easy to dig right in.
    |
    */

    'redis' => [

        'cluster' => false,

        'default' => [
            'host' => env('REDIS_HOST', 'localhost'),
            'password' => env('REDIS_PASSWORD', null),
            'port' => env('REDIS_PORT', 6379),
            'database' => 0,
        ],

        'order' => [
            'host' => env('REDIS_HOST_ORDER', 'localhost'),
            'password' => env('REDIS_PASSWORD_ORDER', null),
            'port' => env('REDIS_PORT_ORDER', 6379),
            'database' => env('REDIS_DATABASE_ORDER', 0),
        ],
        'chapter' => [
            'host' => env('REDIS_HOST_CHAPTER', 'localhost'),
            'password' => env('REDIS_PASSWORD_CHAPTER', null),
            'port' => env('REDIS_PORT_CHAPTER', 6379),
            'database' => env('REDIS_DATABASE_CHAPTER', 0),
        ]

    ],

];
